"""Users (principals) for the WAeUP portal.
"""
import grok
from hurry import yui
from zope.exceptions import DuplicationError
from waeup.authentication import Account
from waeup.interfaces import IUserContainer, IUserAccount
from waeup.viewlets import (LeftSidebar, MainArea, Index, Add, FormWrapMixin,
                            master_template_path)

master_template = grok.PageTemplateFile(master_template_path)


class UserContainer(grok.Container):
    """A container for principals.

    See interfaces.py and users.txt for extensive description.
    """
    grok.implements(IUserContainer)
    grok.require('waeup.manageUsers')

    def addUser(self, name, password, title=None, description=None):
        if title is None:
            title = name
        if description is None:
            description = title
        self[name] = Account(name, password, title, description)

    def addAccount(self, account):
        self[account.name] = account
        
    def delUser(self, name):
        if name in self.keys():
            del self[name]

            
class AddUserForm(grok.AddForm):
    grok.require('waeup.manageUsers')
    grok.context(UserContainer)
    form_fields = grok.AutoFields(IUserAccount)
    label = 'Add user'

    @grok.action('Add user')
    def addFaculty(self, **data):
        #user = Account(**data)
        name = data['name']
        title = data['title']
        description = data['description']
        password = data['password']
        try:
            self.context.addUser(name, password, title=title,
                                 description=description)
        except DuplicationError:
            self.status = Invalid('The userid chosen already exists '
                                  'in the database')
            return
        self.redirect(self.url(self.context))

class AddUser(FormWrapMixin, grok.Viewlet):
    """A viewlet that wraps the `AddUserForm`.
    """
    grok.viewletmanager(MainArea)
    grok.context(UserContainer)
    grok.view(Add)
    grok.require('waeup.manageUsers')

    formview_name = 'adduserform' # The name of the formview we want
                                  # to be rendered in this viewlet.


class UserEditForm(grok.EditForm):
    grok.context(IUserAccount)
    form_fields = grok.AutoFields(IUserAccount)
    grok.require('waeup.manageUsers')    

    @grok.action('Save')
    def save(self, **data):
        self.applyData(self.context, **data)
        self.flash('User was modified.')
        return
    
    @grok.action('Save and return')
    def saveAndReturn(self, **data):
        self.applyData(self.context, **data)
        self.flash('User was modified.')
        self.redirect(self.url(self.context.__parent__))
        return

    @grok.action('Cancel')
    def cancel(self, **data):
        self.redirect(self.url(self.context.__parent__))
        return


class UserEditMain(FormWrapMixin, grok.Viewlet):
    """A viewlet that renders the `UserEditForm`.
    """
    grok.viewletmanager(MainArea)
    grok.context(IUserAccount)
    grok.view(Index)
    grok.require('waeup.manageUsers')
    
    formview_name = 'usereditform' # The name of the formview we want to
                                   # be rendered in this viewlet.
